package com.cpms.project.asset.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.cpms.common.annotation.Log;
import com.cpms.common.core.controller.BaseController;
import com.cpms.common.core.domain.AjaxResult;
import com.cpms.common.enums.BusinessType;
import com.cpms.project.asset.domain.TLeakagePointsRecord;
import com.cpms.project.asset.service.ITLeakagePointsRecordService;
import com.cpms.common.utils.poi.ExcelUtil;
import com.cpms.common.core.page.TableDataInfo;

/**
 * 漏点巡检记录 Controller
 * 
 * @author admin
 * @date 2024-04-01
 */
@RestController
@RequestMapping("/asset/pointRecord")
public class TLeakagePointsRecordController extends BaseController
{
    @Autowired
    private ITLeakagePointsRecordService tLeakagePointsRecordService;

    /**
     * 查询漏点巡检记录 列表
     */
    @GetMapping("/list")
    public TableDataInfo list(TLeakagePointsRecord tLeakagePointsRecord)
    {
        startPage();
        List<TLeakagePointsRecord> list = tLeakagePointsRecordService.selectTLeakagePointsRecordList(tLeakagePointsRecord);
        return getDataTable(list);
    }

    /**
     * 导出漏点巡检记录 列表
     */
    @Log(title = "漏点巡检记录 ", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TLeakagePointsRecord tLeakagePointsRecord)
    {
        List<TLeakagePointsRecord> list = tLeakagePointsRecordService.selectTLeakagePointsRecordList(tLeakagePointsRecord);
        ExcelUtil<TLeakagePointsRecord> util = new ExcelUtil<TLeakagePointsRecord>(TLeakagePointsRecord.class);
        util.exportExcel(response, list, "漏点巡检记录 数据");
    }

    /**
     * 获取漏点巡检记录 详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(tLeakagePointsRecordService.selectTLeakagePointsRecordById(id));
    }

    /**
     * 新增漏点巡检记录 
     */
    @Log(title = "漏点巡检记录 ", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TLeakagePointsRecord tLeakagePointsRecord)
    {
        return toAjax(tLeakagePointsRecordService.insertTLeakagePointsRecord(tLeakagePointsRecord));
    }

    /**
     * 修改漏点巡检记录 
     */
    @Log(title = "漏点巡检记录 ", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TLeakagePointsRecord tLeakagePointsRecord)
    {
        return toAjax(tLeakagePointsRecordService.updateTLeakagePointsRecord(tLeakagePointsRecord));
    }

    /**
     * 删除漏点巡检记录 
     */
    @Log(title = "漏点巡检记录 ", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tLeakagePointsRecordService.deleteTLeakagePointsRecordByIds(ids));
    }
}
